/*
 * File: FunWithText.java
 * ================================================================
 * A program where we have fun with text!
 */
import acm.program.*;

public class FunWithText extends ConsoleProgram {
	/* Time to pause between characters. */
	private static final double PAUSE_TIME = 75;
	
	/**
	 * Prints a string one character at a time.
	 * 
	 * @param text The text to print.
	 */
	private void printCharByChar(String text) {
		for (int i = 0; i < text.length(); i++) {
			char current = text.charAt(i);
			
			/* Print the current character and delay long enough to
			 * have the effect of characters printing one at a time.
			 */
			print(current);
			pause(PAUSE_TIME);
		}
		
		/* Print a newline after everything. */
		println();
	}
	
	/**
	 * Given a string, determines whether that string is a word. This method
	 * doesn't always get the answer right because it uses a <i>heuristic</i>,
	 * an approach that commonly gets the right answer but may be wrong.
	 * <p>
	 * In our case, the heuristic is the following: a word is a nonempty
	 * string made up of letters.
	 * 
	 * @param text The text to analyze.
	 * @return Whether we think it's a word.
	 */
	private boolean isWord(String text) {
		/* The empty string isn't a word. */
		if (text.isEmpty()) return false;
		
		/* Check if any of the characters are not letters. */
		for (int i = 0; i < text.length(); i++) {
			char current = text.charAt(i);
			
			/* If this character isn't a letter, then the input string
			 * isn't a word.
			 * 
			 * TODO: Can you make this more intelligent?
			 */
			if (!Character.isLetter(current)) {
				return false;
			}
		}
		
		/* If all the above checks pass, we think it's a word. */
		return true;
	}
	
	/**
	 * Asks the user to enter a single word, reprompting until they enter
	 * a valid word.
	 * 
	 * @param prompt The prompt to give the user when asking them for a word.
	 * @return The word that the user entered.
	 */
	private String readWord(String prompt) {
		while (true) {
			/* Get some text from the user and hand it back if it's
			 * a word.
			 */
			String text = readLine(prompt);
			if (isWord(text)) return text;
			
			/* What the user entered wasn't a word. Print out an
			 * error message.
			 */
			println("Enter a word, silly!");
		}
	}
	
	/**
	 * Given a string, returns the reverse of that string.
	 * <p>
	 * A note: You can also get the reverse of a string by calling
	 * string.reverse(). We didn't talk about this method in lecture,
	 * but it does indeed exist!
	 * 
	 * @param text The text to reverse.
	 * @return The reverse of that text.
	 */
	private String reverse(String text) {
		String result = "";
		
		/* Walk backwards over the characters in the string, appending
		 * each to the result.
		 */
		for (int i = text.length() - 1; i >= 0; i--) {
			char current = text.charAt(i);
			result += current;
		}
		
		return result;
	}
	
	/**
	 * Given a string, returns whether that string is a palindrome (that is,
	 * whether it's the same forwards and backwards.)
	 * <p>
	 * This code still needs some work - we'll talk about that more next time!
	 * 
	 * @param text The text to test.
	 * @return Whether it's a palindrome.
	 */
	private boolean isPalindrome(String text) {
		return text.equals(reverse(text));
	}
	
	public void run() {
		setFont("UbuntuMono-26-BOLD");
		
		String line = readLine("Enter some text: ");		
		println("Is that a palindrome? " + isPalindrome(line));
	}
}